from __future__ import absolute_import

import socket
import struct
import flask
from octoprint.plugin import StartupPlugin, SettingsPlugin, TemplatePlugin, AssetPlugin, BlueprintPlugin

class WakeOnLanPlugin(StartupPlugin, SettingsPlugin, TemplatePlugin, AssetPlugin, BlueprintPlugin):
    def on_after_startup(self):
        self._logger.info("WakeOnLanPlugin has started")
        
    def get_settings_defaults(self):
        return dict(mac_address="00:00:00:00:00:00")
    
    def get_template_configs(self):
        return [
            dict(type="settings", custom_bindings=False)
        ]
    
    def get_assets(self):
        return dict(
            js=["js/wol.js"],
            css=["css/wol.css"]
        )
    
    def send_wol_packet(self, mac):
        self._logger.info(f"Sending WOL packet to {mac}")
        addr_byte = mac.split(':')
        hw_addr = struct.pack('!6B', *[int(x, 16) for x in addr_byte])
        msg = b'\xff' * 6 + hw_addr * 16
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        s.sendto(msg, ('<broadcast>', 9))
        s.close()
        self._logger.info(f"Sent WOL packet to {mac}")
    
    @BlueprintPlugin.route("/wake", methods=["POST"])
    def wake_endpoint(self):
        mac_address = self._settings.get(["mac_address"])
        self._logger.info(f"Received WOL request for {mac_address}")
        self.send_wol_packet(mac_address)
        return flask.jsonify({"result": "success"})

__plugin_name__ = "WakeOnLanPlugin"
__plugin_implementation__ = WakeOnLanPlugin()
